void sampleDisk( float u1, u2; output float dx, dy ) 
{
	float r, theta;
	// Map uniform random numbers to $[-1,1]^2$
	float sx = 2 * u1 - 1;
	float sy = 2 * u2 - 1;
	// Map square to $(r,\theta)$
	// Handle degeneracy at the origin
	if (sx == 0.0 && sy == 0.0) 
	{
		dx = 0.0;
		dy = 0.0;
	}
	else
	{
		if (sx >= -sy) 
		{
			if (sx > sy) 
			{
				// Handle first region
				r = sx;
				if (sy > 0.0)
					theta = sy/r;
				else
					theta = 8.0 + sy/r;
			}
			else 
			{
				// Handle second region
				r = sy;
				theta = 2.0 - sx/r;
			}
		}
		else 
		{
			if (sx <= sy) 
			{
				// Handle third region
				r = -sx;
				theta = 4.0 - sy/r;
			}
			else 
			{
				// Handle fourth region
				r = -sy;
				theta = 6.0 + sx/r;
			}
		}
		theta *= PI / 4.0;
		dx = r * cos( theta );
		dy = r * sin( theta );
	}
}

surface
cs655surface ( 
	color ambientcolor = 0;
	color specularcolor = 0;
	color reflectedcolor = 0;
	color transmittedcolor = 0;
	color emissivecolor = 0;
	float phong = 1.0;
	float indexofrefraction = 1.0;
	float glossiness = 0;
	float translucency = 0;
	float caustic_power = 2;
	string texturefile = "" )
{
	normal Nn = faceforward (normalize(N),I);
	normal In = normalize( I );

#if defined( BMRT )
	if (isshadowray() == 1)
#else
	string type;
	rayinfo( "type", type );
	
	if ( type == "transmission" ) 
#endif
	{
// No need to run a complex shader for shadows
		float dprod = 1.0 - abs(In.Nn);
		Oi = Os * transmittedcolor * pow(dprod, caustic_power);
		Ci = 1.0;
	} 
	else 
	{
		color reflected;
	     
		if ( comp( reflectedcolor, 0 ) > .001 || comp( reflectedcolor, 1 ) > .001 || comp( reflectedcolor, 2 ) > .001 ) 
		{
			vector reflection = reflect( I, Nn );
			
			if ( glossiness > 0 ) 
			{
				reflection = normalize( reflection );
				
				/* Construct orthogonal components to reflection */
				vector uoffset = normalize ( vector ( 
					zcomp( reflection ) - ycomp( reflection ),
					xcomp( reflection ) - zcomp( reflection ),
					ycomp( reflection ) - xcomp( reflection ) ) );
					
				vector voffset = reflection ^ uoffset;
								
				/* Add a random offset to the smooth reflection vector */
				float dx, dy;
				sampleDisk( random(), random(), dx, dy );
				reflection += glossiness * ( dx * uoffset + dy * voffset );
			}
			reflected = reflectedcolor * color trace( P, reflection );
		}
		
		color transmitted;
		
		if ( comp( transmittedcolor, 0 ) > .001 || comp( transmittedcolor, 1 ) > .001 || comp( transmittedcolor, 2 ) > .001 ) 
		{
			vector refraction = refract( In, Nn, ( N . I ) > 0 ? indexofrefraction : 1 / indexofrefraction ); 
			if ( length( refraction ) != 0.0 )
			{
				if ( translucency > 0 ) 
				{
					refraction = normalize( refraction );
					
					/* Construct orthogonal components to refraction */
					vector uoffset = normalize ( vector ( 
						zcomp( refraction ) - ycomp( refraction ),
						xcomp( refraction ) - zcomp( refraction ),
						ycomp( refraction ) - xcomp( refraction ) ) );
						
					vector voffset = refraction ^ uoffset;
										
					/* Add a random offset to the smooth refraction vector */
					float dx, dy;
					sampleDisk( random(), random(), dx, dy );
					refraction += translucency * ( dx * uoffset + dy * voffset );
				}
				transmitted = transmittedcolor * color trace( P, refraction );
			}
				
		}
		
		color diffused;
		
		if ( texturefile != "" )
			diffused = texture( texturefile );
		else
			diffused = Cs;
	     
		Oi = 1.0;
		Ci = emissivecolor + ambientcolor * ambient() + diffused * diffuse(Nn) + reflected + transmitted
			+ specularcolor * phong( Nn, -In, phong );
	}
}