/* Listing 16.31  Surface shader for a bowling pin*/
/*
 *  pin_color(): Shade a bowling pin with color, labels, and marks. 
 */
#define REDCOLOR 	color (0.6,0.05,0.05)	/* red band on neck */
#define CIRCLECOLOR	color (0.6,0.05,0.05)	/*... for "circle" label */
#define NEUTRALCOLOR 	color (1.0,.8,.4)	/*... for body	*/
#define GRAYCOLOR 	color (.3,.3,.3)	/*... for base	*/
#define BLUECOLOR	color (.0,.0,.9)	/*... for "Brunswick" */

/* Bounding box in texture space for "ABC Circle" label */
#define x1MIN -0.25
#define x1MAX  0.45
#define y1MIN  6.7
#define y1MAX  7.4

/* Bounding box in texture space for "Coated Plastic" label */
#define x2MIN -1.1
#define x2MAX  1.1
#define y2MIN  6.4
#define y2MAX  8.6

/* Bounding box in texture space for "Brunswick B" label */
#define x3MIN -1.3
#define x3MAX  1.3
#define y3MIN  6.0
#define y3MID  6.55
#define y3MAX  8.6

surface
pin_color(
	float	Ka		= .5,
		Kd		= .5,
		Ks		= .8,
		roughness	= .1,
		spin		=  0;
	color	specularcolor   = 1;
	string	texturename     = "marks.txt" )
{
	float	strength;
	point	Nf, V;
	float	x,y,z;
	color	cs ;
	float w;

	Nf = faceforward( normalize(N), I );
	V = normalize(-I);
	x = xcomp(transform("object",P));
	y = ycomp(transform("object",P));							/* Pins are 15" tall 	*/
	z = zcomp(transform("object",P));
	
	if (y <= 0.25) cs = GRAYCOLOR;		/* gray base	*/
	else if (y <= 9.5) cs = NEUTRALCOLOR;	/* neutral body	*/
	else if (y <= 10.5) cs = REDCOLOR;	/* red ring	*/
	else if (y >  12.0) cs = NEUTRALCOLOR;	/* neutral top	*/
	else {			/* between ring and top, create red crown */
		w = (u+.2)*6; w = w-floor(w);
		if (w < .5) 
			if (2*w - 0 > (y-10.5)/(12.0-10.5))
				cs = REDCOLOR;
			else
				cs = NEUTRALCOLOR;
		else
			if (2*w - 1 + (y-10.5)/(12.0-10.5) < 1)
				cs = REDCOLOR;
			else
				cs = NEUTRALCOLOR;
	}

	if (z < 0)	/* on the back side */
		/* Go fetch the "Coated Plastic" label. This is a single-
		 *  channel texture that we want to appear black.	 */
		cs = cs * (1 - float texture("coated.txt",
				(x-x2MIN)/(x2MAX-x2MIN),
				1.0 - (y-y2MIN)/(y2MAX-y2MIN) ));

	else {		/* on the front side */
		/* Go fetch the "ABC circle"  label. This is a single-
		 *  channel texture that we want to appear red.	 */
		strength  = float texture("circle.txt",
				1.0 - (x-x1MIN)/(x1MAX-x1MIN),
				1.0 - (y-y1MIN)/(y1MAX-y1MIN));
		cs = mix(cs,CIRCLECOLOR,strength);
		
		/* Go fetch the "Brunswick B"  label. This is a single-
		 *  channel texture that we want to appear red at the bottom
		 *  and blue at the top.	 */
		strength = float texture("b.txt",
				1.0 - (x-x3MIN)/(x3MAX-x3MIN),
				1.0 - (y-y3MIN)/(y3MAX-y3MIN));
		if (y > y3MID)
			cs = mix(cs,BLUECOLOR,strength);
		else
			cs = mix(cs,REDCOLOR,strength);
	}

	/* 
 	 * Now for the dirt and gouges. Look up in a single channel texture
	 *   for the dirt applied to the surface. Spin the texture differently
	 *   for each pin so that the similarity is not apparent. This same 
	 *   access is made in the displacement shader to align the gouges 
         *   with the dirt. 
	 */
	strength = 1 - float texture(texturename,s+spin,y/15.0);
	cs = cs * strength ;

	/* Now that cs has been set, perform the standard plastic shading 
    	 *  calculation */
	Oi = 1.0;
	Ci = cs * (Ka*ambient() + Kd*diffuse(Nf)) 
		 + Ks*specularcolor*specular(Nf,V,roughness);
}
