/*VSL 1.0.0*************************************************************
 *
 * SURFACE VMarble
 *
 * SYNOPSIS: simple marble
 *
 * AUTHOR: Scott Iverson
 *
 * COPYRIGHT: 2004 SiTex Graphics, Inc.
 *
 * DESCRIPTION:
 *   simple marble pattern based on turbulence\n
 *
 ***********************************************************************/


//PageSize 4000 3000

//Pos 478 74

void PlasticParms_0(
  output float Diffuse; // 0.7 |diffuse coefficient
  output float Specular; // 0.3 |specular coefficient
  output float SpecularRoughness; // 0.1 |surface roughness proportional to highlight size
)
{
}

//Pos 478 189

void ReflectionParms_1(
  output float Reflection; // 0.6 |reflection strength
  output float ReflectionSamples; // 1 |Number of reflection rays when ray tracing
  output float ReflectionBlur; // 0 |blur for reflections
  output string ReflectionName; // "raytrace" |name for environment map or raytrace for ray tracing
  output string ReflectionSpace; // "current" |space for environment lookup
)
{
}

//Pos 52 91

/*DESC
  Set global opacity output variable (Oi)
DESC*/
void SetOpacity_2(
  color Opacity; // Os |opacity
)
{
  extern color Oi;
  
  Oi=Opacity;
}

//Pos 57 36

/*DESC
  Set global output color (Ci)
DESC*/
void SetColor_3(
  color Color; // Cs |color
)
{
  extern color Ci;
  
  Ci=Color;
}

//Pos 462 367

void Marble_4(
  float VeinFraction; // 0.5 |fraction of layer that is vein color
  color VeinColor; // color(0,0,0.2) |color of marble veins
  float VeinSharpness; // 0.5 |contrast adjustment
  float VeinTurbulence; // 1 |amount of turbulence to apply
  float VeinFilterWidth; // 1 |filter width multiplier
  float PatternSize; // 0.25 |width of one layer of marble
  float PatternOrigin[3]; // {0,0,0} |origin of pattern
  float PatternAxis[3]; // {1,0,0} |axis perpendicular to marble layers
  string PatternSpace; // "shader" |coordinate space for shading calculations
  output color Color; // color(1,1,1) |marble color
  output float InMarble; // 1 |fraction in marble
  output float InVein; // 1 |fraction in vein
)
{
  //Pos 307 78
  
  /*DESC
    The object's color attribute value, from shading language global 
    color variable Cs
  DESC*/
  void InColor_0(
    output color InColor; //  |color attribute value
  )
  {
    extern color Cs;
    InColor = Cs;
  }
  
  //Pos 142 123
  
  void MixColors_1(
    color Color0; // color (0,0,0) |
    color Color1; // color(1,1,1) |
    float f; // 0  |
    output color Mix; //  |
  )
  {
    Mix=mix(Color0,Color1,f);
  }
  
  //Pos 316 190
  
  void Marble_2(
    float Sharpness; // 0.5 |contrast adjustment
    float VeinWidth; // 0.5 |vein width
    float Turbulence; // 1 |amount of turbulence to apply
    float Axis[3]; // {1,0,0} |axis perpendicular to marble layers
    float FilterWidth; // 1 |filter width multiplier
    point Position; // P |point at which to calculate turbulence
    output float InMarble; // 1 |fraction in marble
    output float InVein; // 1 |fraction in vein
  )
  {
    //Pos 491 187
    
    /*DESC
      returns a linear combination of two weighted values\n
    DESC*/
    void Combine_0(
      float WeightA; // 1 |first weight
      float ValueA; // 1 |first value
      float WeightB; // 1 |second weight
      float ValueB; // 1 |second value
      output float v; //  |linear combination of v0 and v1
    )
    {
      v = WeightA * ValueA + WeightB * ValueB;
    }
    
    //Pos 642 287
    
    /*DESC
      multiple octaves of noise used to simulate turbulent fluid flow
    DESC*/
    void Turbulence_1(
      point Position; // P |point at which to calculate turbulence
      float Octaves; // 6 |number of octaves of noise
      float Lacunarity; // 2.0 |gap size
      float Gain; // 0.5 |amplitude multiplier for each higher frequency octave
      float FilterWidth; // 1 |filter width multiplier
      output float turb; //  |
    )
    {
      float fw=max(sqrt(area(Position))*FilterWidth,1.0e-6);
      
      float cutoff=min(pow(2,Octaves),0.5/fw);
      float f,fade;
      float g=1;
      float n=Octaves;
      float gavg=0.2;
      
      turb=0;
      
      for (f=1;f<0.5*cutoff;f*=Lacunarity) {
         turb+=g*abs(2*float noise(Position*f)-1);
         g*=Gain; n-=1;
      }
      
      fade=clamp(2*(cutoff-f)/cutoff,0,1);
      turb+=g*(fade*abs(2*float noise(Position*f)-1)+(1-fade)*gavg);
      g*=Gain;
      
      while (n>1) {
        turb+=g*gavg; g*=Gain; n-=1;
      }
    }
    
    //Pos 630 152
    
    /*DESC
      Dot product of A and B
    DESC*/
    void DotProduct_2(
      vector A; // vector(0,0,0) |
      vector B; // vector(0,0,0) |
      output float AdotB; //  |
    )
    {
      AdotB=A.B;
      
    }
    
    //Pos 6 174
    
    void Output_3(
      float InMarble; // 1 |fraction in marble
    )
    {
    }
    
    //Pos 752 141
    
    void Code_4(
      float axis[3]; // {1,1,1} |axis
      output vector vaxis; //  |axis
    )
    {
      vaxis = vector(axis[0],axis[1],axis[2]);
    }
    
    //Pos 120 135
    
    /*DESC
      Remaps an input range to an output range, with option bias and 
      contrast ("gain") adjustment.\n
      \n
      Output values are clamped to the range OutMin..OutMax.
    DESC*/
    void TweakRange_5(
      float In; // s |input value
      float Contrast; // 0.5 |contrast adjustment
      float Bias; // 0.5 |bias between 0 and 1
      float InMin; // 0 |lower bound for input values
      float InMax; // 1 |upper limit for input values
      float OutMin; // 0 |minimum output value
      float OutMax; // 1 |maximum output value
      output float Out; //  |tweaked value
      output float Inverse; //  |mirror function value
    )
    {
      float biasfunc(float b; float v) {
        return pow(v,log(clamp(b,0.001,0.999))/log(0.5));
      }
      
      
      float x = clamp((In-InMin)/(InMax-InMin),0,1);
      
      if (Bias!=0.5) x=biasfunc(Bias,x);
      
      if (Contrast!=0.5) {
         x=(x>0.5) ? biasfunc(1-Contrast,2*x)*0.5 : (2-biasfunc(1-Contrast,2-2*x))*0.5;
      }
      
      Out = float mix(OutMin,OutMax,x);
      Inverse = OutMax+OutMin-Out;
    }
    
    //Pos 12 121
    
    void Output_6(
      float InVein; // 1 |fraction in vein
    )
    {
    }
    
    //Pos 27 17
    //Size 530 55
    
    /*DESC
      Marble pattern produced by adding some turbulence to a simple 
      filtered pulse train representing the layers of material.
    DESC*/
    void _7(
    )
    {
    }
    
    //Pos 296 111
    
    void Parameters_8(
      output float Sharpness; // 0.5 |contrast adjustment
      output float VeinWidth; // 0.5 |vein width
    )
    {
    }
    
    //Pos 609 233
    
    void Parameters_9(
      output float Turbulence; // 1 |amount of turbulence to apply
    )
    {
    }
    
    //Pos 869 138
    
    void Parameters_10(
      output float Axis[3]; // {1,0,0} |axis perpendicular to marble layers
    )
    {
    }
    
    //Pos 778 420
    
    void Parameters_11(
      output float FilterWidth; // 1 |filter width multiplier
    )
    {
    }
    
    //Pos 806 255
    
    void Parameters_12(
      output point Position; // P |point at which to calculate turbulence
    )
    {
    }
    
    //Pos 286 208
    
    /*DESC
      Returns a (filtered) smooth pulse train signal:  \n
      \n
      0 when X<RiseStart\n
      increasing smoothly from 0 to 1 between RiseStart and RiseEnd\n
      remaining at 1 between RiseEnd and FallStart\n
      and decreasing smoothly back to 0 between FallStart and FallEnd\n
    DESC*/
    void SmoothPulseTrain_13(
      float RiseStart; // 0 |start of rising edge of pulse
      float RiseEnd; // .5 |end of rising edge
      float FallStart; // .5 |start of falling edge
      float FallEnd; // 1 |end of falling edge and period of pulse train
      float X; // s |signal source
      float FilterWidth; // 1 |filter width multiplier
      output float Pulse; //  |pulse value
    )
    {
      float intsmoothstep(float e0, e1, x)
      {
         /* smooth step ramp is (3*a*a-2*a*a*a)
            this is its integral */
      
         float integral(float x)
         {
      	return (x*x*x*(1-0.5*x));
         }
      
         return (integral(clamp((x-e0)/max(e1-e0,1.0e-7),0,1))*(e1-e0)+max(0,x-e1));
      }
      extern float du, dv;
      
      float fw = FilterWidth * max(abs(Du(X)*du),abs(Dv(X)*dv));
      
      if (fw<1.0e-7) {
         float x0 = mod(X, FallEnd);
         Pulse=smoothstep(RiseStart,RiseEnd,x0)-smoothstep(FallStart,FallEnd,x0);
      } else {
      
         /* normalize all values to a period of 1 */
      
         float w = fw/FallEnd;
         float e0=RiseStart/FallEnd;
         float e1=RiseEnd/FallEnd;
         float e2=FallStart/FallEnd;
      
         float x0=X/FallEnd-w*0.5;
         float x1=x0+w;
      
         float i0=floor(x0);
         float i1=floor(x1);
      
         float avg=(e1-e0)*0.5+(1-e2)*0.5+(e2-e1);
         x0-=i0; x1-=i1;
      
         Pulse=(avg*(i1-i0)+
                intsmoothstep(e0,e1,x1)-intsmoothstep(e2,1,x1)-
               (intsmoothstep(e0,e1,x0)-intsmoothstep(e2,1,x0)))/w;
      }
    }
    
    //Link 0 1 2 0
    //Link 0 2 9 0
    //Link 0 3 1 0
    //Link 1 0 12 0
    //Link 1 4 11 0
    //Link 2 0 4 0
    //Link 2 1 12 0
    //Link 3 0 5 1
    //Link 4 0 10 0
    //Link 5 0 13 0
    //Link 5 1 8 0
    //Link 5 2 8 1
    //Link 6 0 5 0
    //Link 13 4 0 0
    //Link 13 5 11 0
      float v_0;
      float turb_1;
      float AdotB_2;
      vector vaxis_4;
      float Out_5;
      float Inverse_5;
      float Pulse_13;
      float index;
    
      Turbulence_1(Position,6,2.0,0.5,FilterWidth,turb_1);
      Code_4(Axis,vaxis_4);
      DotProduct_2(vaxis_4,Position,AdotB_2);
      Combine_0(1,AdotB_2,Turbulence,turb_1,v_0);
      SmoothPulseTrain_13(0,.5,.5,1,v_0,FilterWidth,Pulse_13);
      TweakRange_5(Pulse_13,Sharpness,VeinWidth,0,1,0,1,Out_5,Inverse_5);
      InMarble=Inverse_5;
      InVein=Out_5;
  }
  
  //Pos 485 320
  
  /*DESC
    transforms the input point and multiplies by frequency
  DESC*/
  void TransformPoint_3(
    point Pfrom; // P |point
    string FromSpace; // "current" |coordinate space of original point
    string ToSpace; // "shader" |destination coordinate space
    float Frequency; // 1 |scaling factor for p
    float Origin[3]; // {0,0,0} |origin
    output point Pto; //  |transformed point
    output float dPto; //  |filter width at transformed point
  )
  {
    Pto = (transform(FromSpace, ToSpace, Pfrom) - 
           point(Origin[0],Origin[1],Origin[2])) * Frequency;
  }
  
  //Pos 47 123
  
  void Output_4(
    color Color; // color(1,1,1) |marble color
  )
  {
  }
  
  //Pos 144 251
  
  void Output_5(
    float InMarble; // 1 |fraction in marble
    float InVein; // 1 |fraction in vein
  )
  {
  }
  
  //Pos 650 398
  
  void Divide_6(
    float A; // 1 |
    float B; // 1 |
    output float AdivB; //  |
  )
  {
    AdivB=A/B;
  }
  
  //Pos 577 108
  
  void Parameters_7(
    output float VeinFraction; // 0.5 |fraction of layer that is vein color
  )
  {
  }
  
  //Pos 308 132
  
  void Parameters_8(
    output color VeinColor; // color(0,0,0.2) |color of marble veins
  )
  {
  }
  
  //Pos 596 167
  
  void Parameters_9(
    output float VeinSharpness; // 0.5 |contrast adjustment
    output float VeinTurbulence; // 1 |amount of turbulence to apply
    output float VeinFilterWidth; // 1 |filter width multiplier
  )
  {
  }
  
  //Pos 740 429
  
  void PtShadeParms_10(
    output float PatternSize; // 0.25 |width of one layer of marble
  )
  {
  }
  
  //Pos 703 491
  
  void Parameters_11(
    output float PatternOrigin[3]; // {0,0,0} |origin of pattern
  )
  {
  }
  
  //Pos 628 271
  
  void Parameters_12(
    output float PatternAxis[3]; // {1,0,0} |axis perpendicular to marble layers
  )
  {
  }
  
  //Pos 660 338
  
  void PtShadeParms_13(
    output string PatternSpace; // "shader" |coordinate space for shading calculations
  )
  {
  }
  
  //Link 1 0 0 0
  //Link 1 1 8 0
  //Link 1 2 2 0
  //Link 2 0 9 0
  //Link 2 1 7 0
  //Link 2 2 9 1
  //Link 2 3 12 0
  //Link 2 4 9 2
  //Link 2 5 3 0
  //Link 3 2 13 0
  //Link 3 3 6 0
  //Link 3 4 11 0
  //Link 4 0 1 0
  //Link 5 0 2 0
  //Link 5 1 2 1
  //Link 6 1 10 0
    color InColor_0;
    color Mix_1;
    float InMarble_2;
    float InVein_2;
    point Pto_3;
    float dPto_3;
    float AdivB_6;
    float index;
  
    InColor_0(InColor_0);
    Divide_6(1,PatternSize,AdivB_6);
    TransformPoint_3(P,"current",PatternSpace,AdivB_6,PatternOrigin,Pto_3,dPto_3);
    Marble_2(VeinSharpness,VeinFraction,VeinTurbulence,PatternAxis,VeinFilterWidth,Pto_3,InMarble_2,InVein_2);
    MixColors_1(InColor_0,VeinColor,InMarble_2,Mix_1);
    Color=Mix_1;
    InMarble=InMarble_2;
    InVein=InVein_2;
}

//Pos 10 149

void Output_5(
  color __ambient; // color(0,0,0) |ambient color
  color __diffuse; // color(0,0,0) |diffuse component except indirect
  color __diffuse_unshadowed; // color(0,0,0) |unshadowed diffuse
  color __shadow; // color(0,0,0) |diffuse shadow value
  color __indirect; // color(0,0,0) |indirect illumination
  color __specular; // color(0,0,0) |total specular color
  color __reflect; // color(0,0,0) |output reflection
  color __constant; // color(0,0,0) |base color without illumination
)
{
}

//Pos 208 92

/*DESC
  Standard plastic illumination model with reflections including 
  fresnel effects (objects reflect more at grazing angles).\n
DESC*/
void ShinyPlastic_6(
  float Ambient; // 1 |ambient reflectivity
  float Diffuse; // 0.5 |diffuse reflectivity
  float Specular; // 0.5 |specular reflectivity
  float Roughness; // .1 |size of specular highlight
  float Reflection; // 1 |reflection reflectivity
  float ReflectionSamples; // 1 |number of reflection rays when ray tracing
  float ReflectionBlur; // 0 |blur for reflections
  string ReflectionName; // "raytrace" |raytrace or environment map name
  string ReflectionSpace; // "current" |coordinate space for ray tracing or environment map
  normal Normal; // N |shading normal
  color InOpacity; // Os |input opacity
  color SpecularColor; // color(1,1,1) |color for specular highlight and reflections
  color BaseColor; // Cs |diffuse color
  output color Color; //  |plastic color
  output color Opacity; //  |output opacity
  output color __ambient; //  |
  output color __diffuse; //  |
  output color __unshadowed; //  |unshadowed diffuse
  output color __shadow; //  |diffuse shadow value
  output color __indirect; //  |
  output color __specular; //  |
  output color __reflect; //  |
  output color __constant; //  |base color without illumination
  output color __lights[10]; //  |per light output channels
)
{
  color SimpleEnvironment(point P; vector R; string name;
                          string space; float blur; float samples;
                          color weight;)
  {
    point Pndc;
    color Cr=0;
  
    if (name!="")
      if (space=="NDC") {
        Pndc = transform("NDC",P);
        Cr = color texture(name, xcomp(Pndc),ycomp(Pndc), "blur",blur);
      } else if (name=="raytrace") {
        Cr = color environment(name, R, "blur", blur, "samples", samples, "weight", weight);
      } else
        Cr = color environment(name, vtransform(space,R), "blur", blur, "samples", samples);
    
    return Cr;
  }
  
  
  extern vector I;
  vector IN=normalize(I), V=-IN;
  
  normal Nf=faceforward(normalize(Normal),I);
  
  color Crefl=0;
  float fkr=0, fkt=1;
  vector R, T;
    color channels[10];
    float nch=0;
    float i;
  
  
  if (Reflection>0) {
     fresnel(IN,Nf,.667,fkr,fkt,R,T);
     fkt = 1-fkr;
     color weight = fkr * Reflection * SpecularColor;
     Crefl=weight*SimpleEnvironment(P,R,ReflectionName,ReflectionSpace,
               ReflectionBlur,ReflectionSamples,weight);
  }
  
  __ambient = InOpacity * fkt * BaseColor * Ambient * ambient();
  
  color diffuse_weight = InOpacity *fkt * BaseColor * Diffuse;
  
  #ifdef DIFFUSEEXT
  
  __diffuse = diffuse_weight * 
              diffuse(Nf, "indirect", __indirect,
                          "unshadowed", __unshadowed,
                          "shadow", __shadow,
                          "channels", channels,
                          "nchannels", nch);  
  i=0;
  while (i<nch) { __lights[i] = diffuse_weight* channels[i]; i+=1; }
  
  __indirect *= diffuse_weight;
  __unshadowed *= diffuse_weight;
  
  #else
  
  __diffuse = diffuse_weight * diffuse(Nf);
  __unshadowed = __diffuse;
  
  #endif
   
  color specular_weight = InOpacity * Specular * SpecularColor;
#ifdef AIR
  nch = 0;
  __specular = specular_weight * specular(Nf,V,Roughness,
                                            "channels", channels,
                                            "nchannels", nch);
  i=0;
  while (i<nch) { __lights[i] += specular_weight * channels[i]; i+=1; }
#else
    __specular = specular_weight * specular(Nf,V,Roughness );
#endif  
  
  __reflect = InOpacity * Crefl;
  __constant = InOpacity * BaseColor;
  
  Color = __ambient + __diffuse + __specular + __reflect;
  Opacity = InOpacity;
}

//Pos 666 367

void MarbleParms_7(
  output float VeinFraction; // 0.5 |fraction of layer that is vein color
  output color VeinColor; // color(0,0,0.2) |color of marble veins
  output float VeinSharpness; // 0.5 |contrast adjustment
  output float VeinTurbulence; // 1 |amount of turbulence to apply
  output float VeinFilterWidth; // 1 |filter width multiplier
  output float PatternSize; // 0.4 |width of one layer of marble
  output float PatternOrigin[3]; // {0,0,0} |origin of pattern
  output float PatternAxis[3]; // {1,0,0} |axis perpendicular to marble layers
  output string PatternSpace; // "shader" |coordinate space for shading calculations
)
{
}

//Pos 53 365

void Parameters_8(
  color __lights[10]; // {color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0)} |per light output
)
{
}

//Link 2 0 6 1
//Link 3 0 6 0
//Link 4 0 7 0
//Link 4 1 7 1
//Link 4 2 7 2
//Link 4 3 7 3
//Link 4 4 7 4
//Link 4 5 7 5
//Link 4 6 7 6
//Link 4 7 7 7
//Link 4 8 7 8
//Link 5 0 6 2
//Link 5 1 6 3
//Link 5 2 6 4
//Link 5 3 6 5
//Link 5 4 6 6
//Link 5 5 6 7
//Link 5 6 6 8
//Link 5 7 6 9
//Link 6 0 0 0
//Link 6 1 0 0
//Link 6 2 0 1
//Link 6 3 0 2
//Link 6 4 1 0
//Link 6 5 1 1
//Link 6 6 1 2
//Link 6 7 1 3
//Link 6 8 1 4
//Link 6 12 4 0
//Link 8 0 6 10

surface VMarble(
  float Diffuse=0.7;
  float Specular=0.3;
  float SpecularRoughness=0.1;
  float Reflection=0.6;
  float ReflectionSamples=1;
  float ReflectionBlur=0;
  string ReflectionName="raytrace";
  string ReflectionSpace="current";
  float VeinFraction=0.5;
  color VeinColor=color(0,0,0.2);
  float VeinSharpness=0.5;
  float VeinTurbulence=1;
  float VeinFilterWidth=1;
  float PatternSize=0.4;
  float PatternOrigin[3]={0,0,0};
  float PatternAxis[3]={1,0,0};
  string PatternSpace="shader";
  output varying color __ambient=color(0,0,0);
  output varying color __diffuse=color(0,0,0);
  output varying color __diffuse_unshadowed=color(0,0,0);
  output varying color __shadow=color(0,0,0);
  output varying color __indirect=color(0,0,0);
  output varying color __specular=color(0,0,0);
  output varying color __reflect=color(0,0,0);
  output varying color __constant=color(0,0,0);
  output varying color __lights[10]={color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0),color(0,0,0)};
)
 {
  color Color_4;
  float InMarble_4;
  float InVein_4;
  color Color_6;
  color Opacity_6;
  color __ambient_6;
  color __diffuse_6;
  color __unshadowed_6;
  color __shadow_6;
  color __indirect_6;
  color __specular_6;
  color __reflect_6;
  color __constant_6;
  color __lights_6[10];
  float index;

  Marble_4(VeinFraction,VeinColor,VeinSharpness,VeinTurbulence,VeinFilterWidth,PatternSize,PatternOrigin,PatternAxis,PatternSpace,Color_4,InMarble_4,InVein_4);
  ShinyPlastic_6(Diffuse,Diffuse,Specular,SpecularRoughness,Reflection,ReflectionSamples,ReflectionBlur,ReflectionName,ReflectionSpace,N,Os,color(1,1,1),Color_4,Color_6,Opacity_6,__ambient_6,__diffuse_6,__unshadowed_6,__shadow_6,__indirect_6,__specular_6,__reflect_6,__constant_6,__lights_6);
  SetOpacity_2(Opacity_6);
  SetColor_3(Color_6);
  __ambient=__ambient_6;
  __diffuse=__diffuse_6;
  __diffuse_unshadowed=__unshadowed_6;
  __shadow=__shadow_6;
  __indirect=__indirect_6;
  __specular=__specular_6;
  __reflect=__reflect_6;
  __constant=__constant_6;
  for(index=0;index<10;index+=1) __lights[index] = __lights_6[index];
 }
